package spacedata_test

import (
	"bytes"
	"encoding/json"
	"mfs.de/gps"
	"mfs.de/spacedata"
	"strings"
	"testing"
)

func TestLocationProvider_ParseResponse(t *testing.T) {
	jsonString := "{\"timestamp\": 1568153434, \"message\": \"success\", \"iss_position\": {\"longitude\": \"-80.2691\", \"latitude\": \"-45.6056\"}}"
	providerInTesting := spacedata.LocationProvider{}

	expectedCoordinate := gps.CoordinatesWithLatLong(-45.6056, -80.2691)

	result, err := providerInTesting.ParseResponse([]byte(jsonString))
	if err != nil {
		t.Errorf("parsing json sample failed with error: %v", err)
	}

	var tempCoordinate spacedata.Coordinate
	reader := bytes.NewReader(result)
	decoder := json.NewDecoder(reader)
	err = decoder.Decode(&tempCoordinate)
	if err != nil {
		t.Fatalf("decoding people in space returned error: %v", err)
	}
	issCoordinate := tempCoordinate.GpsCoordinate()

	if issCoordinate.Latitude() != expectedCoordinate.Latitude() || issCoordinate.Longitude() != expectedCoordinate.Longitude() {
		t.Errorf("parsing json sample gives wrong output: wantet \n%#v; got \n%#v", expectedCoordinate, issCoordinate)
	}
}

func TestLocationProvider_ParseResponse_JsonSaysNo(t *testing.T) {
	errorMessage := "Nope, Dude."
	jsonString := "{\"timestamp\": 1568153434, \"message\": \"" + errorMessage + "\", \"iss_position\": {\"longitude\": \"-80.2691\", \"latitude\": \"-45.6056\"}}"
	providerInTesting := spacedata.LocationProvider{}

	_, err := providerInTesting.ParseResponse([]byte(jsonString))
	if err == nil || !strings.Contains(err.Error(), errorMessage) {
		t.Errorf("expect error to contain message: %v; got %#v", errorMessage, err)
	}
}
