package spacedata_test

import (
	"bytes"
	"encoding/json"
	"github.com/google/go-cmp/cmp"
	"log"
	"mfs.de/spacedata"
	"strings"
	"testing"
)

func TestPassTimeProvider_ParseResponse(t *testing.T) {
	response := "{\"message\":\"success\",\"request\":{\"altitude\":100,\"datetime\":1568233269,\"latitude\":39.4384,\"longitude\":36.3098,\"passes\":5},\"response\":[{\"duration\":525,\"risetime\":1568244683},{\"duration\":590,\"risetime\":1568250518},{\"duration\":651,\"risetime\":1568256314},{\"duration\":495,\"risetime\":1568262164}]}"

	providerInTesting := spacedata.PassTimeProvider{}

	result, err := providerInTesting.ParseResponse([]byte(response))

	if err != nil {
		t.Errorf("expected to successfully parse response; got error: %v", err)
	}

	if result == nil {
		t.Errorf("expected to successfully parse response; got nil result.")
	}

	expectedRisings := make([]spacedata.Risings, 4)
	expectedRisings[0] = spacedata.Risings{Rising: 1568244683, Duration: 525}
	expectedRisings[1] = spacedata.Risings{Rising: 1568250518, Duration: 590}
	expectedRisings[2] = spacedata.Risings{Rising: 1568256314, Duration: 651}
	expectedRisings[3] = spacedata.Risings{Rising: 1568262164, Duration: 495}

	risingTimes := make([]spacedata.Risings, 0)
	reader := bytes.NewReader(result)
	decoder := json.NewDecoder(reader)
	err = decoder.Decode(&risingTimes)
	if err != nil {
		log.Fatalf("decoding people in space returned error: %v", err)
	}

	for idx, r := range risingTimes {
		if !cmp.Equal(r, expectedRisings[idx]) {
			t.Errorf("expected item at %d to be \n%#v; got \n%#v", idx, expectedRisings[idx], r)
		}
	}
}
func TestPassTimeProvider_ParseResponse_JsonSaysNo(t *testing.T) {
	errorMessage := "Nope, Dude."
	jsonString := "{\"timestamp\": 1568153434, \"message\": \"" + errorMessage + "\", \"iss_position\": {\"longitude\": \"-80.2691\", \"latitude\": \"-45.6056\"}}"
	providerInTesting := spacedata.LocationProvider{}

	_, err := providerInTesting.ParseResponse([]byte(jsonString))
	if err == nil || !strings.Contains(err.Error(), errorMessage) {
		t.Errorf("expect error to contain message: %v; got %#v", errorMessage, err)
	}
}
